@@ -37,6 +37,7 @@ class NapariSTPT:
         self.aligned_4 = None
 
         self.spinN = None
+        self.maxSizeN = None
         self.thresholdN = None
         self.slice_names = None
 
@@ -116,8 +117,8 @@ class NapariSTPT:
                 print("loading C1")
                 volume_1 = (ds.sel(channel=1, type='mosaic',
                             z=0).to_array().data * bscale + bzero)
-                print("cropping C1")
-                volume_1 = self.CropOriginal(volume_1)
+                #print("cropping C1")
+                #volume_1 = self.CropOriginal(volume_1)
                 print("aligning C1")
                 self.aligned_1 = self.Align(
                     volume_1, resolution, output_resolution)
@@ -128,8 +129,8 @@ class NapariSTPT:
                 print("loading C2")
                 volume_2 = (ds.sel(channel=2, type='mosaic',
                             z=0).to_array().data * bscale + bzero)
-                print("cropping C2")
-                volume_2 = self.CropOriginal(volume_2)
+                #print("cropping C2")
+                #volume_2 = self.CropOriginal(volume_2)
                 print("aligning C2")
                 self.aligned_2 = self.Align(
                     volume_2, resolution, output_resolution)
@@ -140,8 +141,8 @@ class NapariSTPT:
                 print("loading C3")
                 volume_3 = (ds.sel(channel=3, type='mosaic',
                             z=0).to_array().data * bscale + bzero)
-                print("cropping C3")
-                volume_3 = self.CropOriginal(volume_3)
+                #print("cropping C3")
+                #volume_3 = self.CropOriginal(volume_3)
                 print("aligning C3")
                 self.aligned_3 = self.Align(
                     volume_3, resolution, output_resolution)
@@ -152,8 +153,8 @@ class NapariSTPT:
                 print("loading C4")
                 volume_4 = (ds.sel(channel=4, type='mosaic',
                             z=0).to_array().data * bscale + bzero)
-                print("cropping C4")
-                volume_4 = self.CropOriginal(volume_4)
+                #print("cropping C4")
+                #volume_4 = self.CropOriginal(volume_4)
                 print("aligning C4")
                 self.aligned_4 = self.Align(
                     volume_4, resolution, output_resolution)
@@ -342,7 +343,9 @@ class NapariSTPT:
 
         return aligned
 
-    def Keep_n_Regions(self):
+
+
+    def Remove_Regions(self, use_size):
 
         output_resolution = float(self.pixel_size.text())
         threshold = float(self.thresholdN.text())
@@ -366,7 +369,10 @@ class NapariSTPT:
                 sizes = stats[:, -1]
                 sizes_sorted = np.sort(sizes, axis=0)
 
-                max_size = sizes_sorted[len(sizes_sorted)-1-keep_n]
+                if use_size:
+                    max_size = float(self.maxSizeN.text())                    
+                else:
+                    max_size = sizes_sorted[len(sizes_sorted)-1-keep_n]
                 for i in range(1, nb_components):
                     if sizes[i] < max_size:
                         volume_z[output == i] = threshold
@@ -402,7 +408,10 @@ class NapariSTPT:
 
                 #regions[z,:,:] = output
 
-                max_size = sizes_sorted[len(sizes_sorted)-1-keep_n]
+                if use_size:
+                    max_size = float(self.maxSizeN.text())                    
+                else:
+                    max_size = sizes_sorted[len(sizes_sorted)-1-keep_n]
                 for i in range(1, nb_components):
                     if sizes[i] < max_size:
                         volume_z[output == i] = threshold
@@ -434,7 +443,10 @@ class NapariSTPT:
                 sizes = stats[:, -1]
                 sizes_sorted = np.sort(sizes, axis=0)
 
-                max_size = sizes_sorted[len(sizes_sorted)-1-keep_n]
+                if use_size:
+                    max_size = float(self.maxSizeN.text())                    
+                else:
+                    max_size = sizes_sorted[len(sizes_sorted)-1-keep_n]
                 for i in range(1, nb_components):
                     if sizes[i] < max_size:
                         volume_z[output == i] = threshold
@@ -463,7 +475,10 @@ class NapariSTPT:
                 sizes = stats[:, -1]
                 sizes_sorted = np.sort(sizes, axis=0)
 
-                max_size = sizes_sorted[len(sizes_sorted)-1-keep_n]
+                if use_size:
+                    max_size = float(self.maxSizeN.text())                    
+                else:
+                    max_size = sizes_sorted[len(sizes_sorted)-1-keep_n]
                 for i in range(1, nb_components):
                     if sizes[i] < max_size:
                         volume_z[output == i] = threshold
@@ -478,6 +493,13 @@ class NapariSTPT:
         #    if self.cb_C2.isChecked():
         #        self.viewer.add_image([self.aligned_2], name='C2 processed', scale=(15,output_resolution,output_resolution), blending='additive', colormap='red')
 
+
+    def Remove_Small_Regions(self):
+        self.Remove_Regions(True)
+
+    def Keep_n_Regions(self):
+        self.Remove_Regions(False)
+
     def Crop(self):
 
         output_resolution = float(self.pixel_size.text())
@@ -571,7 +593,7 @@ class NapariSTPT:
             self.aligned_1 = self.aligned_1[minX_final:maxX_final,
                                             minY_final:maxY_final, minZ_final:maxZ_final]
             self.viewer.layers.remove('C1')
-            self.viewer.add_image([self.aligned_2], name='C1', scale=(
+            self.viewer.add_image([self.aligned_1], name='C1', scale=(
                 15, output_resolution, output_resolution), blending='additive', colormap='bop purple')
         if self.aligned_2 is not None and self.cb_C2.isChecked() and any(i.name == 'C2' for i in self.viewer.layers):
             self.aligned_2 = self.aligned_2[minX_final:maxX_final,
@@ -583,13 +605,13 @@ class NapariSTPT:
             self.aligned_3 = self.aligned_3[minX_final:maxX_final,
                                             minY_final:maxY_final, minZ_final:maxZ_final]
             self.viewer.layers.remove('C3')
-            self.viewer.add_image([self.aligned_2], name='C3', scale=(
+            self.viewer.add_image([self.aligned_3], name='C3', scale=(
                 15, output_resolution, output_resolution), blending='additive', colormap='green')
         if self.aligned_4 is not None and self.cb_C4.isChecked() and any(i.name == 'C4' for i in self.viewer.layers):
             self.aligned_4 = self.aligned_4[minX_final:maxX_final,
                                             minY_final:maxY_final, minZ_final:maxZ_final]
             self.viewer.layers.remove('C4')
-            self.viewer.add_image([self.aligned_2], name='C4', scale=(
+            self.viewer.add_image([self.aligned_4], name='C4', scale=(
                 15, output_resolution, output_resolution), blending='additive', colormap='bop blue')
 
     def SelectFolder(self):
@@ -730,6 +752,14 @@ class NapariSTPT:
             hbox.addStretch(1)
             vbox.addLayout(hbox)
 
+            hbox = QtWidgets.QHBoxLayout()
+            hbox.addWidget(QtWidgets.QLabel("Tissue threshold value:"))
+            self.thresholdN = QtWidgets.QLineEdit("0.3")
+            hbox.addWidget(self.thresholdN)
+            hbox.addStretch(1)
+            vbox.addLayout(hbox)
+
+
             hbox = QtWidgets.QHBoxLayout()
             hbox.addWidget(QtWidgets.QLabel("Number of regions to retain:"))
             self.spinN = QtWidgets.QSpinBox()
@@ -739,17 +769,22 @@ class NapariSTPT:
             vbox.addLayout(hbox)
 
             hbox = QtWidgets.QHBoxLayout()
-            hbox.addWidget(QtWidgets.QLabel("Tissue threshold value:"))
-            self.thresholdN = QtWidgets.QLineEdit("0.1")
-            hbox.addWidget(self.thresholdN)
+            hbox.addWidget(QtWidgets.QLabel("Minimal size:"))
+            self.maxSizeN = QtWidgets.QLineEdit("5000")
+            hbox.addWidget(self.maxSizeN)
             hbox.addStretch(1)
             vbox.addLayout(hbox)
 
+
             hbox = QtWidgets.QHBoxLayout()
             bKeepN = QtWidgets.QPushButton('Show only large regions')
             bKeepN.setCheckable(True)
             bKeepN.clicked.connect(self.Keep_n_Regions)
             hbox.addWidget(bKeepN)
+            bRemoveN = QtWidgets.QPushButton('Remove small regions')
+            bRemoveN.setCheckable(True)
+            bRemoveN.clicked.connect(self.Remove_Small_Regions)
+            hbox.addWidget(bRemoveN)
             bCrop = QtWidgets.QPushButton('Crop')
             bCrop.setCheckable(True)
             bCrop.clicked.connect(self.Crop)
@@ -757,6 +792,8 @@ class NapariSTPT:
             hbox.addStretch(1)
             vbox.addLayout(hbox)
 
+            
+
             bLoad2D = QtWidgets.QPushButton('Load image 2D')
             bLoad2D.setCheckable(True)
             bLoad2D.clicked.connect(self.Load2D)
